/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:block.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include "ed.h"

static long block_start[MAXEDIT];
static long block_end[MAXEDIT];
static int block_flag[MAXEDIT];

static char *tmpnames;

extern char *Realloc();

int
BlockInitOp()
{
	block_start[CurrentFileNo] = 0;
	block_end[CurrentFileNo] = 0;
	block_flag[CurrentFileNo] = INLINE;
}

int
BlockStartOp()
{
	if (block_flag[CurrentFileNo] == INLINE) {
		block_flag[CurrentFileNo] = ONLINE;
		block_start[CurrentFileNo] = GetLineOffset();
		block_end[CurrentFileNo] = GetLineOffset();
		CrtDrawLine(GetRow(), GetLineOffset(), TRUE);
	} else {
		BlockInitOp();
		CrtDrawAll();
	}
}

int
BlockYancOp()
{
	register long tmp_number;
	register FILE *b_fp;
	register EditLine *ed;
	register long b_start;
	register long b_end;

	if (block_flag[CurrentFileNo] != ONLINE) {
		BlockStartOp();
		refresh();
	}

	block_flag[CurrentFileNo] = INLINE;
	block_end[CurrentFileNo] = GetLineOffset();
	unlink(tmpnames);
	if ((tmpnames = tmpnam(NULL)) == NULL) {
		WarningCheck(TMPNAME_NOT_CREATE_MSG);
	} else {
		if ((b_fp = fopen(tmpnames, "w")) == NULL) {
			WarningCheck(TMPFILE_NOT_OPEN_MSG);
		} else {
			b_start = min(block_start[CurrentFileNo], block_end[CurrentFileNo]);
			b_end = max(block_start[CurrentFileNo], block_end[CurrentFileNo]);
			ed = (EditLine *)GetList(b_start);
			for (tmp_number = b_start; tmp_number <= b_end; tmp_number++) {
				fprintf(b_fp, "%s\n", ed->buffer);
				ed = ed->next;
			}
			fclose(b_fp);
			CrtDrawAll();
		}
	}
}

int
BlockDeleteOp()
{
	register long tmp_number;
	register EditLine *ed;
	register long b_start;
	register long b_end;

	flushinp();
	if (block_flag[CurrentFileNo] != ONLINE) {
		BlockStartOp();
		refresh();
	}
	block_flag[CurrentFileNo] = INLINE;
	block_end[CurrentFileNo] = GetLineOffset();
	b_start = min(block_start[CurrentFileNo], block_end[CurrentFileNo]);
	b_end = max(block_start[CurrentFileNo], block_end[CurrentFileNo]);
	for (tmp_number = b_start; tmp_number <= b_end; tmp_number++) {
		ed = (EditLine *)GetList(b_start);
		if (b_start < GetLastNumber()) {
			DeleteList(ed);
		} else if (Realloc(ed, "") == NULL) {
			WarningCheck(MEMORY_ERR_MSG);
			return;
		}
	}
	if (b_end - b_start < GetRowWidth()) {
		for (tmp_number = b_start; tmp_number < b_end; tmp_number++) {
			CursorUpOp();
		}
	} else {
		CursorSetRow(GetMinRow());
		LineOffsetSet(b_start);
	}
	/*FILE Change FLAG*/
	SetFileChangeFlag();
	CrtDrawAll();
}

int
BlockMoveOp()
{
	register long tmp_number;
	register EditLine *ed;
	register FILE *b_fp;
	register long b_start;
	register long b_end;

	if (block_flag[CurrentFileNo] != ONLINE) {
		BlockStartOp();
		refresh();
	}
	block_flag[CurrentFileNo] = INLINE;
	block_end[CurrentFileNo] = GetLineOffset();
	unlink(tmpnames);
	if ((tmpnames = tmpnam(NULL)) == NULL) {
		WarningCheck(TMPNAME_NOT_CREATE_MSG);
	} else {
		if ((b_fp = fopen(tmpnames, "w")) == NULL) {
			WarningCheck(TMPFILE_NOT_OPEN_MSG);
		} else {
			b_start = min(block_start[CurrentFileNo], block_end[CurrentFileNo]);
			b_end = max(block_start[CurrentFileNo], block_end[CurrentFileNo]);
			/*Save Tmp file*/
			ed = (EditLine *)GetList(b_start);
			for (tmp_number = b_start; tmp_number <= b_end; tmp_number++) {
				fprintf(b_fp, "%s\n", ed->buffer);
				ed = ed->next;
			}
			fclose(b_fp);

			for (tmp_number = b_start; tmp_number <= b_end; tmp_number++) {
				ed = (EditLine *)GetList(b_start);
				if (b_start < GetLastNumber()) {
					DeleteList(ed);
				} else {
					if (Realloc(ed, "") == NULL) {
						WarningCheck(MEMORY_ERR_MSG);
						return;
					}
				}
			}
			if (b_end - b_start < GetRowWidth()) {
				for (tmp_number = b_start; tmp_number < b_end; tmp_number++) {
					CursorUpOp();
				}
			} else {
				CursorSetRow(GetMinRow());
				LineOffsetSet(b_start);
			}
			/*FILE Change FLAG*/
			SetFileChangeFlag();
			CrtDrawAll();
		}
	}
}

int
BlockPasteOp()
{
	register long tmp_number;
	char tmp_buff[MAXEDITLINE+2];
	register FILE *b_fp;
	register EditLine *ed;
	int len;

	if ((b_fp = fopen(tmpnames, "r")) == NULL) {
		system_msg(TMPFILE_NOT_OPEN_MSG);
	} else {
		tmp_number = 0;
		while (fgets(tmp_buff, MAXEDITLINE+1, b_fp) != NULL) {
			len = strlen(tmp_buff);
			if (len > 0 && tmp_buff[len-1] == '\n') {
				tmp_buff[len-1] = '\0';
			}
			if ((ed = (EditLine *)MakeLine(tmp_buff)) == NULL) {
				WarningCheck(MEMORY_ERR_MSG);
				break;
			} else {
				InsertLine(GetList(GetLineOffset()+tmp_number-1), ed);
				tmp_number++;
				/*FILE Change FLAG*/
				SetFileChangeFlag();
			}
		}
		fclose(b_fp);
		CrtDrawAll();
	}
}

long
GetBlockStart()
{
	return block_start[CurrentFileNo];
}

long
GetBlockEnd()
{
	return block_end[CurrentFileNo];
}

int
GetBlockFlg()
{
	return block_flag[CurrentFileNo];
}

char *
GetBlockNames()
{
	return tmpnames;
}

int
SetBlockEnd()
{
	block_end[CurrentFileNo] = GetLineOffset();
}

int
SetBlockEndTo(EndOffset)
register long EndOffset;
{
	block_end[CurrentFileNo] = EndOffset;
}

/*-------------------------------------------------------------------
	Block one line Yanc
-------------------------------------------------------------------*/
int
BlockOneLineYancOp()
{
	BlockStartOp();
	garbagedlines(stdscr, GetRow(), 1);
	refresh();
	BlockYancOp();
}
