/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:cursor.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include <ctype.h>
#include "ed.h"


static int cursor_row[MAXEDIT];
static int cursor_col[MAXEDIT];
static int v_scroll[MAXEDIT];
static int redraw_flg;
VOID
CursorSetRow(row)
register int row;
{
	if (GetMinRow() <= row && row <= GetMaxRow()-1) {
		cursor_row[CurrentFileNo] = row;
	} else if (GetMinRow() > row) {
		cursor_row[CurrentFileNo] = GetMinRow();
	} else if (GetMaxRow()-1 < row) {
		cursor_row[CurrentFileNo] = GetMaxRow()-1;
	}
}

VOID
CursorSetCol(col)
register int col;
{
	if (GetMinCol() < col && col < GetMaxCol()) {
		cursor_col[CurrentFileNo] = col;
	} else if (col <= GetMinCol()) {
		cursor_col[CurrentFileNo] = GetMinCol()+1;
	} else if (GetMaxCol() <= col) {
		cursor_col[CurrentFileNo] = GetMaxCol();
	}
}

VOID
CursorSetScroll(scroll)
register int scroll;
{
	if (scroll >= 0) {
		v_scroll[CurrentFileNo] = scroll;
	} else {
		v_scroll[CurrentFileNo] = 0;
	}
}

int
GetRow()
{
	return cursor_row[CurrentFileNo];
}

int
GetCol()
{
	return cursor_col[CurrentFileNo];
}

int
GetScroll()
{
	return v_scroll[CurrentFileNo];
}

VOID
CursorInit()
{
	CursorSetRow(GetMinRow());
	CursorSetCol(GetMinCol());
	CursorSetScroll(0);
}

VOID
CursorSetByColOffset()
{
	register int ColOffset;
	register int crtwidth;

	ColOffset = GetOffsetOfBuffer(GetLineOffset());
	crtwidth = GetColWidth()-NumWidth;

	if (ColOffset != GetCol()+GetScroll()) {
		if (GetScroll() < ColOffset && ColOffset < GetScroll()+crtwidth) {
			CursorSetCol(ColOffset-GetScroll());
		} else if (ColOffset <= GetScroll()) {
			CursorSetScroll(ColOffset-1);
			CursorSetCol(GetMinCol());
		} else if (GetScroll()+crtwidth <= ColOffset) {
			CursorSetScroll(ColOffset - crtwidth);
			CursorSetCol(crtwidth);
		}
	}
}

VOID
CursorUp()
{
	if (GetRow() > GetMinRow()) {
		CursorSetRow(GetRow()-1);
	} else if (GetLineOffset()-1 > 0) {
		CrtScrollDown();
	}
}

VOID
CursorDown()
{
	register long LastOffset;
	LastOffset = GetLastNumber();
	if (GetRow() < GetMaxRow()-1 && GetLineOffset() < LastOffset) {
		CursorSetRow(GetRow()+1);
	} else if (GetLineOffset() < LastOffset) {
		CrtScrollUp();
	}
}

VOID
CursorLeft()
{
	register int tmp_scroll;

	if (GetCol() > GetMinCol()) {
		CursorSetCol(GetCol()-1);
		tmp_scroll = GetScroll();
		CursorSetByColOffset();
		if (tmp_scroll != GetScroll()) {
			CursorSetScroll(GetScroll()-TabStop);
			CursorSetByColOffset();
		}
	}
}

VOID
CursorRight()
{
	if (GetCol() < (GetColWidth()-NumWidth)) {
		CursorSetCol(GetCol()+1);
		CursorSetByColOffset();
	} else {
		CursorSetScroll(GetScroll()+TabStop);
		CursorSetByColOffset();
	}
}

int
GetCorrectColumn(LineNumber)
register long LineNumber;
{
	register int i;
	register int cols;
	register int Correct;
	register int nextTabStop;
	register int strlength;
	register EditLine *ed;

	ed = (EditLine *)GetList(LineNumber);
	cols = GetCol()+GetScroll();
	strlength = strlen(ed->buffer);

	for (i = 0, Correct = 1; Correct < cols && i < strlength; i++) {
		if (ed->buffer[i] == '\t') {
			nextTabStop = ((Correct-1)/TabStop+1)*TabStop+1;
			if (cols < nextTabStop) {
				return Correct;
			}
			Correct = nextTabStop;
		} else if (iscntrl((unsigned char)(ed->buffer[i]))) {
			Correct += 2;
		} else {
			Correct++;
		}
	}


	if (CursorMode == FREE) {
		while (Correct < cols) {
			Correct++;
		}
	}
	if (IsThisKanjiPosition(i-1, ed->buffer) == TRUE) {
		Correct--;
	}
	return Correct;
}

VOID
CursorMove()
{
	int putColumn;
	int tmpCol;
	int tmpScroll;

	tmpCol = GetCol();
	tmpScroll = GetScroll();

	putColumn = GetCorrectColumn(GetLineOffset());

	OffsetSetByColum(GetLineOffset());

	if (tmpScroll != GetScroll() || redraw_flg == TRUE) {
		CrtDrawAll();
	}
	if (tmpScroll != GetScroll()) {
		redraw_flg = TRUE;
	} else {
		redraw_flg = FALSE;
	}
	putColumn = putColumn - GetScroll();
	move(GetRow(), putColumn+NumWidth-1);
	CursorSetCol(tmpCol);
	CursorSetScroll(tmpScroll);
}

/*-------------------------------------------------------------------
	Get Cursor Column by Buffer Offset
-------------------------------------------------------------------*/
int
ColumnGetByOffset(offset)
int offset;
{
	register int i;
	register int cols;
	register int strlength;
	register EditLine *ed;

	ed = (EditLine *)GetList(GetLineOffset());
	strlength = strlen(ed->buffer);

	for (i = 0, cols = 1; i < offset; i++) {
		if (i < strlength) {
			if (ed->buffer[i] == '\t') {
				cols = (cols +TabStop)/TabStop*TabStop;
			} else if (iscntrl((unsigned char)(ed->buffer[i]))) {
				cols += 2;
			} else {
				cols++;
			}
		} else {
			cols++;
		}
	}
	return cols;
}
