/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:file.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include <sys/types.h>
#include <sys/stat.h>
#include "ed.h"
#include "pathname.h"

extern char *LsMenu();

VOID
CheckFileAccess(fileNo)
int fileNo;
{
	struct stat sbuf;
	if (CreateTime[fileNo] != -1) {
		stat(EditPathName[fileNo], &sbuf);
		if (sbuf.st_ctime != CreateTime[fileNo]) {
			system_msg(TARGET_FILE_ACCESSED_MSG);
		}
	}
}

int
CheckWildCard(buffer)
char *buffer;
{
	if (strpbrk(buffer, "*?[]") == NULL) {
		return FALSE;
	} else {
		return TRUE;
	}
}

int
CheckMultiFile(buffer)
char *buffer;
{
	if (strpbrk(buffer, " ") == NULL) {
		return FALSE;
	} else {
		return TRUE;
	}
}

int
isPathname(pathname)
char *pathname;
{
	if (strpbrk(pathname, "/~") == NULL) {
		return FALSE;
	} else {
		return TRUE;
	}
}


/*--------------------------------------------------------------------
	WILD CARD
--------------------------------------------------------------------*/
typedef struct _namelist {
	char *name;
	struct _namelist *next;
} namelist;

namelist *
AppendName(list, name)
namelist *list;
char *name;
{
	namelist *new;

	if ((new = (namelist *)malloc(sizeof(namelist))) == NULL) {
		return list;
	}

	if ((new -> name = (char *)malloc(strlen(name)+1)) == NULL) {
		free(new);
		return list;
	}
	strcpy(new -> name, name);
	new -> next = list;
	return new;
}

namelist *
FreeName(list)
namelist *list;
{
	namelist *next;

	free(list -> name);
	next = list -> next;
	free(list);
	return next;
}

VOID
FreeNameAll(list)
namelist *list;
{
	while (list) {
		list = FreeName(list);
	}
}

namelist *
PickupName(path, list)
char *path;
namelist *list;
{
	if (list == NULL) {
		return NULL;
	}

	strncpy(path, list -> name, MAXPATHLEN);
	path[MAXPATHLEN] = '\0';
	list = FreeName(list);

	return list;
}

namelist *
SortName(list)
namelist *list;
{
	namelist *lp1, *lp2;
	char *cp;

	for (lp1 = list; lp1 && lp1 -> next; lp1 = lp1 -> next) {
		for (lp2 = lp1 -> next; lp2; lp2 = lp2 -> next) {
			if (strcmp(lp1 -> name, lp2 -> name) > 0) {
				cp = lp1 -> name;
				lp1 -> name = lp2 -> name;
				lp2 -> name = cp;
			}
		}
	}
	return list;
}

namelist *
MatchWildCard(list, fixed, lower)
namelist *list;
char *fixed;
char *lower;
{
	DIR *dirp;
	struct dirent *dp;
	reg_t *regexp;
	char wild[MAXNAMLEN+1];
	char *cp, *cnvwild;
	int i, tmp;

	for (i = 0; i < MAXNAMLEN && lower[i] && lower[i] != '/'; i++) {
		wild[i] = lower[i];
	}
	wild[i] = '\0';
	if (i == 0) {
		if (lower[i] == '/') {
			strcat(fixed, "/");
			list = MatchWildCard(list, fixed, &lower[i+1]);
		} else {
			if (fixed[0] != '\0' && strcmp(fixed, "/") != 0) {
				strcat(fixed, "/");
			}
			list = AppendName(list, fixed);
		}
		return list;
	}

	if ((cnvwild = cnvregexp(wild, 1)) == NULL) {
		return list;
	}

	if (fixed[0] == '\0') {
		dirp = opendir(".");
	} else {
		dirp = opendir(fixed);
	}
	if (dirp == NULL) {
		return list;
	}

	while ((dp = readdir(dirp)) != NULL) {
		if (strcmp(dp -> d_name, ".") == 0
		|| strcmp(dp -> d_name, "..") == 0) {
			if (strcmp(dp -> d_name, wild) != 0) {
				continue;
			}
		} else {
			regexp = regexp_init(cnvwild);
			tmp = regexp_exec(regexp, dp -> d_name);
			regexp_free(regexp);
			if (!tmp) {
				continue;
			}
		}

		cp = fixed+strlen(fixed);
		if (fixed[0] != '\0' && strcmp(fixed, "/") != 0) {
			strcat(fixed, "/");
		}
		strcat(fixed, dp -> d_name);
		if (lower[i] == '/') {
			list = MatchWildCard(list, fixed, &lower[i+1]);
		} else {
			list = AppendName(list, fixed);
		}
		*cp = '\0';
	}
	closedir(dirp);
	free(cnvwild);

	return list;
}

namelist *
GetWildCard(wildcard)
char *wildcard;
{
	namelist *list;
	char fullpath[MAXPATHLEN+1];
	char *cp;

	for (cp = wildcard; *cp == ' ' || *cp == '\t'; cp++);
	if (*cp == '\0' || (cp = _evalpath(cp, cp + strlen(cp))) == NULL) {
		return NULL;
	}

	fullpath[0] = '\0';
	list = MatchWildCard(NULL, fullpath, cp);
	if (DirectorySort_flag == TRUE) {
		SortName(list);
	}
	free(cp);

	return list;
}

char *
ReadWildCard(path, wildcard)
char *path;
char *wildcard;
{
	static namelist* list = (namelist *)-1;

	if (wildcard == NULL) {
		FreeNameAll(list);
		list = (namelist *)-1;
		return NULL;
	}

	if (list == (namelist *)-1) {
		list = GetWildCard(wildcard);
	}
	if (list == NULL) {
		list = (namelist *)-1;
		return NULL;
	}
	list = PickupName(path, list);
	return path;
}

char *WildMenu(filename, list)
char *filename;
namelist *list;
{
	register int num;
	struct stat sbuf;
	char tmpbuff[MAXFILEMENU][MAXPATHLEN+1];
	char *buffer[MAXFILEMENU];
	char *cp;
	register int m_select;

	m_select = 0;
	getcwd(filename, MAXPATHLEN);
	move(0, 0);
	clrtoeol();
	CrtNormal(stdscr);
	CrtReverse(stdscr);
	printw("Path : ");
	widthputs(filename, GetMaxCol()-7);

	for (num = 0; num < MAXFILEMENU && list; num++) {
		while (list) {
			list = PickupName(tmpbuff[num], list);
			if (!(stat(tmpbuff[num], &sbuf))
			&& (sbuf.st_mode & S_IFMT) == S_IFREG) {
				break;
			}
		}
		buffer[num] = tmpbuff[num];
	}
	if (num <= 0) {
		return NULL;
	}
	NumMenu_open(GetMinRow(), GetMinCol(), buffer, num);
	m_select = NumMenu_select(GetMinRow(), GetMinCol(), buffer, num, m_select);
	if (m_select == -1) {
		*filename = '\0';
	} else if (tmpbuff[m_select][0] != '/') {
		strcat(filename, "/");
		strcat(filename, tmpbuff[m_select]);
	} else {
		strcpy(filename, tmpbuff[m_select]);
		cp = strrchr(tmpbuff[m_select], '/');
		if (cp == NULL) {
			chdir("/");
		} else {
			*cp = '\0';
			chdir(tmpbuff[m_select]);
		}
	}

	return filename;
}


/*--------------------------------------------------------------------
	/home/nebashi/test	->	test
--------------------------------------------------------------------*/
char *
CutFilename(filename, pathname)
char *filename;
char *pathname;
{
	register int i = 0;
	register int j = 0;
	register int strlength;
	char *fname;
	strlength = strlen(pathname);
	if (pathname[0] == '~') {
		for (i = 0; i < strlength; i++) {
			if (pathname[i] == '/') {
				break;
			}
		}
	}
	for (j = 0; i < strlength; i++, j++) {
		filename[j] = pathname[i];
	}
	filename[j] = '\0';

	if ((fname=strrchr(filename, '/')) != NULL) {
		strcpy(filename, fname + 1);
	}
	return filename;
}

/*--------------------------------------------------------------------
	/home/nebashi/test	->	/home/nebashi
--------------------------------------------------------------------*/
char *
CutPathname(path, pathname)
char *path;
char *pathname;
{
	register int i;
	register int ignore;
	register int strlength;

	strcpy(path, pathname);
	strlength = strlen(path);
	ignore = 0;
	if (pathname[0] == '~') {
		for (i = 1, ignore = 1; i < strlength; i++) {
			if (pathname[i] == '/') {
				break;
			}
		}
	}
	if (isPathname(path) == TRUE) {
		for (i = strlength-1; i >= ignore; i--) {
			if (path[i] != '/') {
				continue;
			}
			path[i] = '\0';
			break;
		}
	}
	strcat(path, "/");
	return path;
}

VOID
SetFileChangeFlag()
{
	if (FileChg_flag[CurrentFileNo] == FALSE) {
		FileChg_flag[CurrentFileNo] = TRUE;
		system_guide();
	}
}

VOID
ResetFileChangeFlag()
{
	if (FileChg_flag[CurrentFileNo] == TRUE) {
		FileChg_flag[CurrentFileNo] = FALSE;
		system_guide();
	}
}

int
CheckFileOpened(ino, fullpath)
int ino;
char *fullpath;
{
	int i;

	if (ino == -1) {
		for (i = FILE_START; i < MAXEDIT; i++) {
			if (EditPathName[i][0] != '\0') {
				if (strcmp(fullpath, EditPathName[i]) == 0) {
					return i;
				}
			}
		}
	} else {
		for (i = FILE_START; i < MAXEDIT; i++) {
			if (EditPathName[i][0] != '\0') {
				if (ino == FileInode[i]) {
					return i;
				}
			}
		}
	}
	return -1;
}

/*-------------------------------------------------------------------
	FILE Save
-------------------------------------------------------------------*/
int
filesave(filename)
char *filename;
{
	register FILE *fp;
	register EditLine *ed;
	struct stat sbuf;
	int res;

	char backpath[MAXPATHLEN+1];
	char buffer[MAXEDITLINE+1];
	char savefile[MAXPATHLEN+1];

	strcpy(savefile, filename);
	if (FileChg_flag[CurrentFileNo] == TRUE) {
		if (strcmp(savefile, "untitled.txt") == 0) {
			*savefile = '\0';
			if (RenameFile(CurrentFileNo, savefile) == FALSE) {
				return FALSE;
			} else {
				strcpy(savefile, EditPathName[CurrentFileNo]);
			}
		}
		if (access(savefile, F_OK) == 0 && access(savefile, W_OK) == -1) {
			CrtDrawAll();
			WhileKbHit(DONT_WRITE_FILE_MSG);
			return FALSE;
		} else {
			if (CreateTime[CurrentFileNo] != -1) {
				stat(savefile, &sbuf);
				if (sbuf.st_ctime != CreateTime[CurrentFileNo]) {
					CrtDrawAll();
					res = WarningCheck(TARGET_FILE_CHANGED_MSG);
					if (res != TRUE) {
						return FALSE;
					}
				}
			}
			if (access(savefile, F_OK) == 0 && BackFile_flag == TRUE) {
				/*Make Back up FILE Name*/
				strcpy(backpath, savefile);
				/*Add ".bk"*/
				strcat(backpath, ".bak");
				/*Cleat Back up File*/
				rename(savefile, backpath);
			}
			if ((fp = fopen(savefile, "w")) != NULL) {
				sprintf(buffer, "%s %s .. ", SAVEING_MSG, savefile);
				ed = (EditLine *)GetTop();

				while (ed->next != NULL) {
					fputs(ed->buffer, fp);
					fputc('\n', fp);
					ed = ed->next;
				}
				if (ed->buffer[0] != '\0') {
					fputs(ed->buffer, fp);
				}
				fclose(fp);

				sprintf(buffer, "%s %s .. ", SAVESUCCESS_MSG, savefile);
				system_msg(buffer);
				/*FILE Change FLAG*/
				ResetFileChangeFlag();
				return TRUE;
			} else {
				CrtDrawAll();
				WhileKbHit(DONT_WRITE_FILE_MSG);
				return FALSE;
			}
		}
	}
	return TRUE;
}

VOID
SetPermissionFlg(filename)
char *filename;
{
	register int permission;

	permission = access(filename, W_OK);
	Permission[CurrentFileNo] = permission;
}

int
fileopen(filename)
char *filename;
{
	register FILE *fp;
	register EditLine *ed;
	char buffer[MAXEDITLINE+2];
	struct stat sbuf;
	int iNo;
	int cTime;
	int len, eof;

	if (access(filename, F_OK) == -1) {
		buffer[0] = '\0';
		if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
			WhileKbHit(MEMORY_ERR_MSG);
		} else {
			AppendLast(ed);
		}
		sprintf(buffer, "%s [ %s ]", NEWFILE_MSG, filename);
		system_msg(buffer);
		CreateTime[CurrentFileNo] = -1;
		FileInode[CurrentFileNo] = -1;
	} else if (access(filename, R_OK) == -1) {
		WhileKbHit(PERMISSION_MSG);
		return FALSE;
	} else {
		if ((fp = fopen(filename, "r")) != NULL) {
			if (!(iNo = cTime = stat(filename, &sbuf))) {
				iNo = sbuf.st_ino;
				cTime = sbuf.st_ctime;
			}

			CreateTime[CurrentFileNo] = cTime;
			FileInode[CurrentFileNo] = iNo;

			eof = 0;
			sprintf(buffer, "%s  %s ", READING_MSG, filename);
			system_msg(buffer);
			while (fgets(buffer, MAXEDITLINE+1, fp) != NULL) {
				len = strlen(buffer);
				eof = 0;
				if (len > 0 && buffer[len-1] == '\n') {
					buffer[len-1] = '\0';
					eof = 1;
				}
				if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
					WhileKbHit(MEMORY_ERR_MSG);
					break;
				} else {
					AppendLast(ed);
				}

			}
			fclose(fp);

			if (eof == 1) {
				buffer[0] = '\0';
				if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
					WhileKbHit(MEMORY_ERR_MSG);
				} else {
					AppendLast(ed);
				}
			}

			if (!GetLastNumber()) {
				if (ed = (EditLine *)MakeLine(""))
					AppendLast(ed);
				else WhileKbHit(MEMORY_ERR_MSG);
			}
			SetPermissionFlg(filename);
		} else {
			return FALSE;
		}
	}
	return TRUE;
}

/*-------------------------------------------------------------------
	Get FILE No
		Error -1
-------------------------------------------------------------------*/
int
GetFileNo(filename)
char *filename;
{
	register int i;
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			if (strcmp(EditPathName[i], filename) == 0) {
				return i;
			}
		}
	}
	return -1;
}

/*-------------------------------------------------------------------
	FILE NAME Buffer ALL Clear
-------------------------------------------------------------------*/
VOID
FileNameInit()
{
	register int i;

	for (i = FILE_START; i < MAXEDIT; i++) {
		EditPathName[i][0] = '\0';
	}
}

/*-------------------------------------------------------------------
	Set FILE NAME
		Set FILE Name to FILE NAME Buffer and Return No.
-------------------------------------------------------------------*/
int
SetFileName(pathname, filename)
char *pathname;
char *filename;
{
	register int i;

	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] == '\0') {
			strcpy(EditPathName[i], pathname);
			strcpy(EditFileName[i], filename);
			return i;
		}
	}
	return -1;
}

int
GetEditFileNum()
{
	register int i;
	register int num;
	for (i = FILE_START, num = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			num++;
		}
	}
	return num;
}

/*-------------------------------------------------------------------
	FILE NAME Buffer Clear
-------------------------------------------------------------------*/
VOID
ResetFileName(no)
register int no;
{
	EditPathName[no][0] = '\0';
}

int
FindOutNextFile(no)
int no;
{
	int i;
	for (i = no; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0' && i != CurrentFileNo) {
			return i;
		}
	}
	for (i = FILE_START; i < no; i++) {
		if (EditPathName[i][0] != '\0' && i != CurrentFileNo) {
			return i;
		}
	}
	return -1;
}

/*-------------------------------------------------------------------
	Get BackFile No
-------------------------------------------------------------------*/
int
GetBackFile(current_no)
int current_no;
{
	register int i;

	if (EditPathName[BackFileNo][0] != '\0') {
		/*If Exists BackFile then return This No*/
		if (BackFileNo == current_no) {
			for (i = FILE_START; i < MAXEDIT; i++) {
				if (EditPathName[i][0] != '\0' && i != current_no) {
					return i;
				}
			}
			return -1;
		} else {
			return BackFileNo;
		}
	} else {
		/*If Not Exists BackFIle then Find out*/
		return FindOutNextFile(current_no);
	}
}

/*-------------------------------------------------------------------
	Initialize before FILE Open
-------------------------------------------------------------------*/
VOID
FileStartInit()
{
	CursorInit();
	BufferOffsetInit();
	LineOffsetInit();
	InitLine();
	ResetFileChangeFlag();
	BlockInitOp();
}

char *
CreateFileName(fullpath, inName)
char *fullpath;
char *inName;
{
	namelist *plist, *flist;
	register int i;
	char path[MAXPATHLEN+1];
	char fname[MAXNAMLEN+1];
	char *cutName;
	char inFile[MAXPATHLEN+1];
	struct stat sbuf;
	char tmpDirectory[MAXPATHLEN+1];
	char Directory[MAXPATHLEN+1];

	GetEditFileNum();
	getcwd(tmpDirectory, MAXPATHLEN);

	strcpy(inFile, ".");
	for (i = 0; i < strlen(inName); i++) {
		if (inName[i] != ' ') {
			strcpy(inFile, inName);
			break;
		}
	}
	cutName = strtok(inFile, " \t");
	do {
		if (isPathname(cutName) == TRUE) {
			CutPathname(path, cutName);
			CutFilename(fname, cutName);
		} else {
			strcpy(path, "./");
			strcpy(fname, cutName);
		}

		plist = GetWildCard(path);
		while (plist) {
			plist = PickupName(path, plist);
			chdir(path);
			getcwd(Directory, MAXPATHLEN);		/* Full Directory */
			flist = GetWildCard(fname);
			if (flist == NULL) {
				sprintf(fullpath, "%s/%s", Directory, fname);
			} else if (flist -> next == NULL) {
				PickupName(fname, flist);
				if (stat(fname, &sbuf) == 0
				&& (sbuf.st_mode & S_IFMT) == S_IFDIR) {
					FreeNameAll(plist);
					return NULL;
				} else {
					sprintf(fullpath, "%s/%s", Directory, fname);
				}
		 	} else {
				while (flist) {
					flist = PickupName(fname, flist);
					if (stat(fname, &sbuf) == 0
					&& (sbuf.st_mode & S_IFMT) != S_IFDIR) {
						FreeNameAll(plist);
						FreeNameAll(flist);
						return NULL;
					}
				}
		 	}
			chdir(tmpDirectory);
		}
	} while ((cutName = strtok(NULL, " \t")) != NULL);
	if (*fullpath != '\0') {
		return fullpath;
	} else {
		return NULL;
	}
}

int
RenameFile(current_no, filename)
register int current_no;
char *filename;
{
	register int i;
	char fullpath[MAXPATHLEN+1];
	char InputFileName[MAXEDITLINE+1];
	struct stat sbuf;
	int iNo;
	int cTime;

	if (*filename == '\0') {
		if (HisGets(InputFileName, RENAME_MSG, FOPEN_SYSTEM) == NULL) {
			/*BelowLineDraw();*/
			return FALSE;
		}
	} else {
		strcpy(InputFileName, filename);
	}
	InputFileName[MAXPATHLEN] = '\0';
	if (CreateFileName(fullpath, InputFileName) == NULL) {
		return FALSE;
	}

	if (access(fullpath, F_OK) != -1) {
		if (UserIsYes(FILE_EXIST_MSG) == TRUE) {
			strcpy(EditPathName[current_no], fullpath);
			CutFilename(EditFileName[current_no], fullpath);
			SetFileChangeFlag();
			SetPermissionFlg(EditPathName[current_no]);
			if (!(iNo = cTime = stat(EditPathName[CurrentFileNo], &sbuf))) {
				iNo = sbuf.st_ino;
				cTime = sbuf.st_ctime;
			}
			CreateTime[current_no] = cTime;
			FileInode[current_no] = iNo;
			return TRUE;
		} else {
			return FALSE;
		}
	}
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			if (strcmp(EditPathName[i], fullpath) == 0) {
				if (UserIsYes(THIS_FILE_OPENED_MSG) == TRUE) {
					strcpy(EditPathName[current_no], fullpath);
					CutFilename(EditFileName[current_no], fullpath);
					SetFileChangeFlag();
					SetPermissionFlg(EditPathName[current_no]);
					if (!(iNo = cTime = stat(EditPathName[CurrentFileNo], &sbuf))) {
						iNo = sbuf.st_ino;
						cTime = sbuf.st_ctime;
					}
					CreateTime[current_no] = cTime;
					FileInode[current_no] = iNo;
					return TRUE;
				} else {
					return FALSE;
				}
			}
		}
	}
	strcpy(EditPathName[current_no], fullpath);
	CutFilename(EditFileName[current_no], fullpath);
	SetFileChangeFlag();
	SetPermissionFlg(EditPathName[current_no]);
	if (!(iNo = cTime = stat(EditPathName[CurrentFileNo], &sbuf))) {
		iNo = sbuf.st_ino;
		cTime = sbuf.st_ctime;
	}
	CreateTime[current_no] = cTime;
	FileInode[current_no] = iNo;
	return TRUE;
}

/*-------------------------------------------------------------------
	toggle file
-------------------------------------------------------------------*/
VOID
toggleFileOp()
{
	register int tmpNo;

	if ((tmpNo = GetBackFile(CurrentFileNo)) != -1) {
		BackFileNo = CurrentFileNo;
		CurrentFileNo = tmpNo;
		CrtDrawAll();
		CheckFileAccess(CurrentFileNo);
	}
}

/*-------------------------------------------------------------------
	Set File
-------------------------------------------------------------------*/
VOID
ChangeFileOp()
{
	register int i;
	register int j;
	register int file_no;

	if ((file_no = SelectFileMenu()) != -1) {
		for (i = FILE_START, j = FILE_START; i < MAXEDIT; i++) {
			if (EditPathName[i][0] != '\0') {
				if (j == file_no) {
					break;
				}
				j++;
			}
		}

		BackFileNo = CurrentFileNo;
		CurrentFileNo = i;
	}
	CrtDrawAll();
	CheckFileAccess(CurrentFileNo);
}

int
FileOpenWithFullPath(fpath, filename)
char *fpath;
char *filename;
{
	register int file_no;
	register int getNo;
	char pathname[MAXPATHLEN+1];
	char fname[MAXNAMLEN+1];
	struct stat sbuf;
	int fileNo;
	int iNo;

	strcpy(pathname, fpath);
	strcpy(fname, filename);
	if ((file_no = GetFileNo(pathname)) == -1) {
		if (!(iNo = stat(pathname, &sbuf))) {
			iNo = sbuf.st_ino;
		}
		if ((sbuf.st_mode & S_IFMT) == S_IFDIR) {
			if (*LsMenu(pathname, pathname) == '\0') {
				if (GetEditFileNum() == FILE_START) {
					EndProcedure();
				} else {
					CrtDrawAll();
					return FALSE;
				}
			}
			CutFilename(fname, pathname);
		}
		if (!(iNo = stat(pathname, &sbuf))) {
			iNo = sbuf.st_ino;
		}
		if ((fileNo = CheckFileOpened(iNo, pathname)) == -1) {
			if ((getNo = SetFileName(pathname, fname)) == -1) {
				WhileKbHit(DONT_OPEN_MORE_FILE_MSG);
				return FALSE;
			} else {
				BackFileNo = CurrentFileNo;
				CurrentFileNo = getNo;
			}
			FileStartInit();
			fileopen(EditPathName[CurrentFileNo]);
		} else {
			BackFileNo = CurrentFileNo;
			CurrentFileNo = fileNo;
		}
	} else {
		/*FILE Change*/
		if (CurrentFileNo != file_no) {
			BackFileNo = CurrentFileNo;
			CurrentFileNo = file_no;
		}
	}
	return TRUE;
}

/*-------------------------------------------------------------------
	FILE open
-------------------------------------------------------------------*/
int FileOpenOne(path)
char *path;
{
	char *fname;
	struct stat sbuf;
	char fullpath[BUFSIZ];
	int fileNo;
	int iNo;

	if (path[0] == '\0') {
		return FALSE;
	}
	if (!(iNo = stat(path, &sbuf))) {
		iNo = sbuf.st_ino;
		if ((sbuf.st_mode & S_IFMT) != S_IFREG) {
			return FALSE;
		}
	}

	if ((fname = strrchr(path, '/')) == NULL) {
		strcpy(fullpath, "./");
		fname = path;
	} else {
		strncpy(fullpath, path, (++fname) - path);
		fullpath[fname - path] = '\0';
	}
	chdir(fullpath);
	getcwd(fullpath,BUFSIZ);
	strcat(fullpath, "/");
	strcat(fullpath, fname);

	if ((fileNo = CheckFileOpened(iNo, fullpath)) >= 0) {
		BackFileNo = CurrentFileNo;
		CurrentFileNo = fileNo;
	} else if ((fileNo = SetFileName(fullpath, fname)) < 0) {
		WhileKbHit(DONT_OPEN_MORE_FILE_MSG);
		return -1;
	} else {
		BackFileNo = CurrentFileNo;
		CurrentFileNo = fileNo;
		FileStartInit();
		fileopen(EditPathName[CurrentFileNo]);
	}

	return TRUE;
}

int
FileOpenOp(inName, wild)
char *inName;
int wild;
{
	namelist *list;
	register int i;
	char fullpath[MAXPATHLEN+1];
	char *cutName;
	char inFile[MAXPATHLEN+1];
	char tmpDirectory[MAXPATHLEN+1];

	GetEditFileNum();
	getcwd(tmpDirectory, MAXPATHLEN);

	strcpy(inFile, ".");
	for (i = 0; i < strlen(inName); i++) {
		if (inName[i] != ' ') {
			strcpy(inFile, inName);
			break;
		}
	}

	cutName = strtok(inFile, " \t");
	do {
		if (wild == 0) {
			strcpy(fullpath, cutName);
			if (FileOpenOne(fullpath) < 0) {
				return FALSE;
			}
		} else {
			list = GetWildCard(cutName);
			if (list == NULL || list -> next == NULL) {
				if (list == NULL) {
					strcpy(fullpath, cutName);
				} else {
					PickupName(fullpath, list);
				}
				if (FileOpenOne(fullpath) < 0) {
					return FALSE;
				}
			} else {
				if (wild == 1) {
					while (list) {
						list = PickupName(fullpath, list);
						if (FileOpenOne(fullpath) < 0) {
							return FALSE;
						}
					}
				} else if (WildMenu(fullpath, list) != NULL) {
					if (FileOpenOne(fullpath) < 0) {
						return FALSE;
					}
				}
			}
		}
		chdir(tmpDirectory);
	} while ((cutName = strtok(NULL, " \t")) != NULL);

	if (GetEditFileNum() == FILE_START) {
		system_msg(NOT_OPEN_OR_CREATE_MSG);
		return FALSE;
	}
	return TRUE;
}

/*-------------------------------------------------------------------
	Input FILE NAME and open
-------------------------------------------------------------------*/
VOID
SetName_FileOpenOp()
{
	char fname[MAXEDITLINE+1];

	if (HisGets(fname, OPEN_MSG, FOPEN_SYSTEM) != NULL) {
		system_msg(WAITING_MSG);
		fname[MAXPATHLEN] = '\0';
		if (FileOpenOp(fname, 2) == FALSE) {
			return;
		}
		CrtDrawAll();
		return;
	}
	/*BelowLineDraw();*/
	return;
}

/*-------------------------------------------------------------------
	ALL FILE SAVE and END
-------------------------------------------------------------------*/
int
AllSave_End()
{
	register int i;

	SaveProfile();
	CrtDrawAll();
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			CurrentFileNo = i;
			if (filesave(EditPathName[i]) == FALSE) {
				return FALSE;
			}
			ResetFileName(i);
			FreeLine();
			InitLine();
		}
	}
	EndProcedure();

	return TRUE;
}

/*-------------------------------------------------------------------
	Current FILE Save
-------------------------------------------------------------------*/
int
CurrentFileSave()
{
	int res;
	struct stat sbuf;
	int iNo;
	int cTime;

	SaveProfile();
	if ((res = filesave(EditPathName[CurrentFileNo])) == FALSE) {
		return FALSE;
	}

	if (!(iNo = cTime = stat(EditPathName[CurrentFileNo], &sbuf))) {
		iNo = sbuf.st_ino;
		cTime = sbuf.st_ctime;
	}
	CreateTime[CurrentFileNo] = cTime;
	FileInode[CurrentFileNo] = iNo;

	return res;
}

/*-------------------------------------------------------------------
	Current FILE KILL
-------------------------------------------------------------------*/
int
CurrentFileKill()
{
	register int res;

	SaveProfile();

	if (FileChg_flag[CurrentFileNo] == TRUE) {
		res = WarningCheck(THIS_FILE_CHANGED_MSG);
		if (res == TRUE) {
			if (filesave(EditPathName[CurrentFileNo]) == FALSE) {
				return FALSE;
			}
		} else if (res == ESCAPE) {
			return FALSE;
		}
	}
	CrtDrawAll();
	ResetFileName(CurrentFileNo);
	FreeLine();
	InitLine();
	if (EditPathName[BackFileNo][0] != '\0') {
		CurrentFileNo = BackFileNo;
		CrtDrawAll();
		BackFileNo = FindOutNextFile(CurrentFileNo);
		CheckFileAccess(CurrentFileNo);
		return TRUE;
	} else {
		CurrentFileNo = FindOutNextFile(CurrentFileNo);
		if (FILE_START <= CurrentFileNo) {
			CrtDrawAll();
			BackFileNo = FindOutNextFile(CurrentFileNo);
			CheckFileAccess(CurrentFileNo);
			return TRUE;
		}
	}
	EndProcedure();

	return TRUE;
}

/*-------------------------------------------------------------------
	ALL FILE KILL and END
-------------------------------------------------------------------*/
int
AllFileSave()
{
	register int i;
	register int tmpNo1;
	register int tmpNo2;
	struct stat sbuf;
	int iNo;
	int cTime;

	tmpNo1 = CurrentFileNo;
	tmpNo2 = BackFileNo;
	SaveProfile();
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			CurrentFileNo = i;
			if (filesave(EditPathName[CurrentFileNo]) == TRUE) {
				if (!(iNo = cTime = stat(EditPathName[CurrentFileNo], &sbuf))) {
					iNo = sbuf.st_ino;
					cTime = sbuf.st_ctime;
				}
				CreateTime[CurrentFileNo] = cTime;
				FileInode[CurrentFileNo] = iNo;
			}
		}
	}
	CurrentFileNo = tmpNo1;
	BackFileNo = tmpNo2;
	CrtDrawAll();
	return TRUE;
}

/*-------------------------------------------------------------------
	ALL FILE KILL and END
-------------------------------------------------------------------*/
int
AllKill_End()
{
	register int i;
	register int res;

	SaveProfile();
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			if (FileChg_flag[i] == TRUE) {
				res = WarningCheck(EXIST_CHANGED_FILE_MSG);
				if (res == TRUE) {
					AllSave_End();
				} else if (res == FALSE) {
					break;
				} else {
					return FALSE;
				}
			}
		}
	}
	CrtDrawAll();
	for (i = FILE_START; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			CurrentFileNo = i;
			ResetFileName(CurrentFileNo);
			FreeLine();
			InitLine();
		}
	}
	EndProcedure();

	return TRUE;
}

VOID
FreshStartFile()
{
	register int res;
	register long lineOffset;
	register long Bottom;
	res = TRUE;
	if (FileChg_flag[CurrentFileNo] == TRUE) {
		res = WarningCheck(ARE_YOU_SURE_MSG);
	}
	if (res == TRUE) {
		lineOffset = GetLineOffset();
		Bottom = GetLineOffset() + (GetRowWidth() - GetRow())-1;
		FreeLine();
		InitLine();
		FileStartInit();
		fileopen(EditPathName[CurrentFileNo]);
		JmpNumber(Bottom);
		JmpNumber(lineOffset);

		CrtDrawAll();
	}
}

/*-------------------------------------------------------------------
	FILE MENU
-------------------------------------------------------------------*/
static int m_select = 0;
int
SelectFileMenu()
{
	register int i;
	register int j;
	register int m;
	register int res;
	char tmpbuff[MAXEDIT][MAXLINESTR+1];
	char *buffer[MAXEDIT];

	for (i = FILE_START, j = 0, m = 0; i < MAXEDIT; i++) {
		if (EditPathName[i][0] != '\0') {
			if (i < CurrentFileNo) {
				m++;
			}
			sprintf(tmpbuff[j], "%-.*s %1s ", GetColWidth()-6, EditFileName[i], (FileChg_flag[i] ? "*":""));
			buffer[j] = tmpbuff[j];
			j++;
		}
	}

	NumMenu_open(GetMinRow(), GetMinCol(), buffer, j);
	res = NumMenu_select(GetMinRow(), GetMinCol(), buffer, j, m);
	if (res == -1) {
		return -1;
	} else {
		m_select = res+FILE_START;
		return m_select;
	}
}

static int file_menu_res = 0;
VOID
FileMenu()
{
	register int tmpflag;
	char tmpbuff[20][MAXLINESTR+1];
	char *buffer[20];
	char buff[20];
	char InputFileName[MAXEDITLINE+1];
	register int res;
	int iNo;
	int cTime;

		sprintf(tmpbuff[0], "%s", MENU_OPEN_MSG);
		sprintf(tmpbuff[1], "%s", MENU_SAVE_MSG);
		sprintf(tmpbuff[2], "%s", MENU_SAVEAS_MSG);
		sprintf(tmpbuff[3], "%s", MENU_SAVE_AND_DONE_MSG);
		sprintf(tmpbuff[4], "%s", MENU_SAVE_PROFILE_MSG);
		sprintf(tmpbuff[5], "%s", MENU_RENAME_MSG);
		sprintf(tmpbuff[6], "%s", MENU_ABANDON_CF_MSG);
		sprintf(tmpbuff[7], "%s", MENU_ABANDON_AF_MSG);
		sprintf(tmpbuff[8], "%s", MENU_REFRESH_CF_MSG);
		sprintf(tmpbuff[9], "%s", MENU_ESCAPE_SHELL_MSG);

		buffer[0] = tmpbuff[0];
		buffer[1] = tmpbuff[1];
		buffer[2] = tmpbuff[2];
		buffer[3] = tmpbuff[3];
		buffer[4] = tmpbuff[4];
		buffer[5] = tmpbuff[5];
		buffer[6] = tmpbuff[6];
		buffer[7] = tmpbuff[7];
		buffer[8] = tmpbuff[8];
		buffer[9] = tmpbuff[9];

		NumMenu_open(GetMinRow(), GetMinCol(), buffer, 10);
		res = NumMenu_select(GetMinRow(), GetMinCol(), buffer, 10, file_menu_res);
		switch(res) {
		case 0:
			SetName_FileOpenOp();
			break;
		case 1:
			CurrentFileSave();
			break;
		case 2:
			if (HisGets(InputFileName, SAVE_AS_MSG, FOPEN_SYSTEM) != NULL) {
				tmpflag = FileChg_flag[CurrentFileNo];
				cTime = CreateTime[CurrentFileNo];
				iNo = FileInode[CurrentFileNo];

				CreateTime[CurrentFileNo] = -1;
				FileInode[CurrentFileNo] = -1;

				SetFileChangeFlag();
				filesave(InputFileName);
				CreateTime[CurrentFileNo] = cTime;
				FileInode[CurrentFileNo] = iNo;
				FileChg_flag[CurrentFileNo] = tmpflag;
			} else {
				/*BelowLineDraw();*/
			}
			break;
		case 3:
			AllSave_End();
			break;
		case 4:
			SaveProfile();
			break;
		case 5:
			buff[0] = '\0';
			RenameFile(CurrentFileNo, buff);
			break;
		case 6:
			CurrentFileKill();
			break;
		case 7:
			AllKill_End();
			break;
		case 8:
			FreshStartFile();
			break;
		case 9:
			ShellEscape("");
			break;
		default:
			CrtDrawAll();
			return;
		}
		file_menu_res = res;
		CrtDrawAll();
}

/*
char *
GetFileStr()
{
	return InputFileName;
}

VOID
SetFileStr(buffer)
char *buffer;
{
	strcpy(InputFileName, buffer);
}
*/
