/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:list.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include "ed.h"

#define	xalloc(t, n)	((t*)malloc((unsigned)sizeof(t)*(n)))
#define	xrealloc(p, t, n)	((t*)realloc((char *)p, (unsigned)sizeof(t)*(n)))
#define	xfree(p)	(free((char *)p))

/*Base Pointer*/
static EditLine Baseline[MAXEDIT];
static EditLine *LastLine[MAXEDIT];
static long LastOffset[MAXEDIT];
/*-------------------------------------------------------------------
	Initiarize Structure
-------------------------------------------------------------------*/
VOID
FreeLine()
{
	EditLine	*p, *q;

	for (p = Baseline[CurrentFileNo].next; p != NULL; p = q) {
		q = p->next;
		p->next = NULL;
		p->behind = NULL;
		xfree(p->buffer);
		xfree(p);
	}
}

/*-------------------------------------------------------------------

-------------------------------------------------------------------*/
char *
Realloc(li, n_buffer)
register EditLine *li;
char *n_buffer;
{
	char *str;

	if ((str = xrealloc(li->buffer , char, strlen(n_buffer)+2)) == (char *)NULL) {
		return NULL;
	}

	strcpy(str, n_buffer);

	li->buffer = str;
	return str;
}

/*-------------------------------------------------------------------

-------------------------------------------------------------------*/
VOID
InitLine()
{
	Baseline[CurrentFileNo].behind = NULL;
	Baseline[CurrentFileNo].next = NULL;
	LastLine[CurrentFileNo] = &Baseline[CurrentFileNo];
	LastOffset[CurrentFileNo] = 0;
}

/*-------------------------------------------------------------------
	Get Top of List
-------------------------------------------------------------------*/
EditLine *
GetTop()
{
	return Baseline[CurrentFileNo].next;
}

/*-------------------------------------------------------------------
	Get Last of List
-------------------------------------------------------------------*/
EditLine *
GetLast()
{
	return LastLine[CurrentFileNo];
}

/*-------------------------------------------------------------------
	Make Line
-------------------------------------------------------------------*/
EditLine *
MakeLine(buffer)
char *buffer;
{
	char *str;
	register EditLine *pli;

	if ((pli = xalloc(EditLine, 1)) == NULL) {
		return NULL;
	}

	if ((str = xalloc(char, strlen(buffer)+2)) == NULL) {
		return NULL;
	}

	strcpy(str, buffer);
	pli->buffer = str;
	pli->mark = FALSE;
	return pli;
}

/*-------------------------------------------------------------------
	Add Line at Top
-------------------------------------------------------------------*/
VOID
AppendTop(li)
register EditLine *li;
{
	register EditLine *top, *next;

	top = GetTop();
	top = top->behind;
	next = top->next;
	top->next = li;
	li->behind = top;
	li->next = next;
	next->behind = li;
	LastOffset[CurrentFileNo]++;
}

/*-------------------------------------------------------------------
	Add Line at Last
-------------------------------------------------------------------*/
VOID
AppendLast(li)
EditLine	*li;
{
	register EditLine *last;

	last = GetLast();
	last->next = li;
	li->behind = last;
	li->next = NULL;
	LastOffset[CurrentFileNo]++;
	LastLine[CurrentFileNo] = li;
}

/*-------------------------------------------------------------------
	Insert Line at List
-------------------------------------------------------------------*/
VOID
InsertLine(bli, li)
register EditLine *bli;
register EditLine *li;
{
	EditLine	*nextli;

	if (bli->next == NULL) {
		AppendLast(li);
	} else {
		nextli = bli->next;
		li->behind = bli;
		li->next = bli->next;
		bli->next = li;
		nextli->behind = li;
		LastOffset[CurrentFileNo]++;
	}
}

/*-------------------------------------------------------------------
	Delete line by list
-------------------------------------------------------------------*/
VOID
DeleteList(li)
register EditLine *li;
{
	register EditLine *behind, *next;

	behind = li->behind;
	next = li->next;
	if (li->next != NULL) {
		next->behind = behind;
	} else {
		LastLine[CurrentFileNo] = behind;
	}
	behind->next = next;
	xfree(li->buffer);
	xfree(li);
	LastOffset[CurrentFileNo]--;
}

/*-------------------------------------------------------------------

-------------------------------------------------------------------*/
EditLine *
GetList(o_number)
register long o_number;
{
	long	num;
	register EditLine *last;


	num = 0;
	last = &Baseline[CurrentFileNo];

	while (last->next != NULL) {
		if (o_number == num++) {
			return last;
		}
		last = last->next;
	}
	return last;
}

long
GetNumber(li)
register EditLine *li;
{
	register long num;
	register EditLine *last;

	num = 1;
	last = &Baseline[CurrentFileNo];

	while (last->next != NULL) {
		if (li == last->next) {
			return num;
		}
		last = last->next;
		num++;
	}
	return num;
}

long
GetTopNumber()
{
	return 1;
}

long
GetLastNumber()
{
	return LastOffset[CurrentFileNo];
}
