/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:menu.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include <ctype.h>
#include "ed.h"

static int menu_cursor;
static int item_number;
static int putitem;
static int maxitem;
static int maxlength;

VOID
NumMenu_open(row, col, buffer, nitem)
register int row;
register int col;
char **buffer;
register int nitem;
{
	register int i;
	register int rowOffset;
	register int colOffset;

	rowOffset = row;
	colOffset = col;

	move(row, col);
	maxitem = GetRowWidth()-2;
	if (nitem < GetRowWidth()-2-row) {
		putitem = nitem;
	} else if (nitem < GetRowWidth()-2) {
		putitem = nitem;
		rowOffset = GetRowWidth() - nitem -2;
	} else {
		putitem = GetRowWidth()-2;
		rowOffset = GetMinRow();
	}

	/*Get Max strings*/
	for (i = 0, maxlength = 0; i < nitem; i++) {
		if (maxlength < strlen(buffer[i])) {
			maxlength = strlen(buffer[i]);
		}
	}
	if (GetColWidth() < maxlength+4) {
		maxlength = GetColWidth()-4;
	}

	if (GetMaxCol() < colOffset + maxlength+4) {
		colOffset = GetMaxCol() - (maxlength+4)-1;
	}

	CrtNormal(stdscr);
	move(rowOffset, colOffset);
	CrtReverse(stdscr);
	addch(ACS_ULCORNER);
	for (i = 0; i < maxlength+2; i++) {
		addch(ACS_HLINE);
	}
	addch(ACS_URCORNER);
	if (stable_standout) {
		move(rowOffset, colOffset);
		CrtReverse(stdscr);
		move(rowOffset, colOffset+2-1+maxlength+3);
	}
	CrtNormal(stdscr);

	for (i = 0; i < putitem; i++) {
		move(rowOffset+i+1, colOffset);
		CrtReverse(stdscr);
		printw("%c ", ACS_VLINE);
		widthputs(buffer[i], maxlength);
		printw(" %c", ACS_VLINE);
		if (stable_standout) {
			move(rowOffset+i+1, colOffset);
			CrtReverse(stdscr);
			move(rowOffset+i+1, colOffset+2-1+maxlength+3);
		}
		CrtNormal(stdscr);
	}
	move(rowOffset+putitem+1, colOffset);
	CrtReverse(stdscr);
	addch(ACS_LLCORNER);
	for (i = 0; i < maxlength+2; i++) {
		addch(ACS_HLINE);
	}
	addch(ACS_LRCORNER);
	if (stable_standout) {
		move(rowOffset+putitem+1, colOffset);
		CrtReverse(stdscr);
		move(rowOffset+putitem+1, colOffset+2-1+maxlength+3);
	}
	CrtNormal(stdscr);
	move(rowOffset+putitem, colOffset+2-1+maxlength+1);
}

VOID
NumMenu_itemRedraw(row, col, item_num, buffer)
register int row;
register int col;
register int item_num;
char **buffer;
{
	register int i;

	for (i = 0; i < putitem; i++) {
		if (stable_standout) {
			CrtReverse(stdscr);
		}
		move(row+1+i, col+2-1);
		CrtReverse(stdscr);
		addch(' ');
		widthputs(buffer[i+item_num], maxlength);
		CrtNormal(stdscr);
	}
}

/*--------------------------------------------------------------------
	Menu Cursor On
--------------------------------------------------------------------*/
VOID
NumMenu_cursoron(row, col, buffer)
register int row;
register int col;
char **buffer;
{
	if (stable_standout) {
		CrtReverse(stdscr);
		move(row+1+menu_cursor, col+2-1);
		CrtNormal(stdscr);
		move(row+1+menu_cursor, col+2-1+maxlength+2);
		CrtReverse(stdscr);
		move(row+1+menu_cursor, col+2-1+maxlength+1);
	} else {
		move(row+1+menu_cursor, col+2-1);
		CrtNormal(stdscr);
		addch(' ');
		widthputs(buffer[item_number], maxlength);
	}
	CrtNormal(stdscr);
}

/*--------------------------------------------------------------------
	Menu Cursor Off
--------------------------------------------------------------------*/
VOID
NumMenu_cursoroff(row, col, buffer)
register int row;
register int col;
char **buffer;
{
	if (stable_standout) {
		CrtReverse(stdscr);
	}
	move(row+1+menu_cursor, col+2-1);
	CrtReverse(stdscr);
	addch(' ');
	widthputs(buffer[item_number], maxlength);
	CrtNormal(stdscr);
}

VOID
NumMenu_cursorup(row, col, nitem, buffer)
register int row;
register int col;
register int nitem;
char **buffer;
{
	if (menu_cursor > 0) {
		menu_cursor--;
		item_number--;
	} else if (item_number > 0) {
		item_number--;
		NumMenu_itemRedraw(row, col, item_number, buffer);
	}
}

VOID
NumMenu_cursordown(row, col, nitem, buffer)
register int row;
register int col;
register int nitem;
char **buffer;
{
	if (menu_cursor < putitem-1 && item_number < nitem-1) {
		menu_cursor++;
		item_number++;
	} else if (item_number < nitem-1) {
		item_number++;
		NumMenu_itemRedraw(row, col, item_number-putitem+1, buffer);
	}
}

int
m_search(c, nitem, buffer)
register int c;
register int nitem;
char **buffer;
{
	register int i;
	register int it;
	int c2;

	it = item_number+1;
	for (i = 0; i < nitem; i++, it++) {
		if (nitem <= it) {
			it = 0;
		}
		c2 = buffer[it][0];
		if (islower(c2)) c2 += 'A' - 'a';
		if (c2 == c) {
			return it;
		}
	}
	return item_number;
}

int
NumMenu_select(row, col, buffer, nitem, start_num)
register int row;
register int col;
char **buffer;
register int nitem;
register int start_num;
{
	register int i;
	register int c;
	register int num;
	register int rowOffset;
	register int colOffset;

	rowOffset = row;
	colOffset = col;

	if (nitem < GetRowWidth()-2-row) {
		putitem = nitem;
	} else if (nitem < GetRowWidth()-2) {
		rowOffset = GetRowWidth() - nitem - 2;
	} else {
		rowOffset = GetMinRow();
	}

	if (GetMaxCol() < colOffset + maxlength+4) {
		colOffset = GetMaxCol() - (maxlength+4)-1;
	}
	for (menu_cursor = 0, item_number = 0; item_number < start_num;) {
		NumMenu_cursordown(rowOffset, colOffset, nitem, buffer);
	}
	NumMenu_cursoron(rowOffset, colOffset, buffer);
	num = 0;
	for (;;) {
		c = inkey();
			switch(GetSyStemKey(c)) {
			case KEY_UP:
				NumMenu_cursoroff(rowOffset, colOffset, buffer);
				NumMenu_cursorup(rowOffset, colOffset, nitem, buffer);
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_DOWN:
				NumMenu_cursoroff(rowOffset, colOffset, buffer);
				NumMenu_cursordown(rowOffset, colOffset, nitem, buffer);
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_PPAGE:
				num = putitem-1;
				NumMenu_cursoroff(rowOffset, colOffset, buffer);
				for (i = 0; i < num; i++) {
					NumMenu_cursorup(rowOffset, colOffset, nitem, buffer);
				}
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_NPAGE:
				num = putitem-1;
				NumMenu_cursoroff(rowOffset, colOffset, buffer);
				for (i = 0; i < num; i++) {
					NumMenu_cursordown(rowOffset, colOffset, nitem, buffer);
				}
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_HOME:
				menu_cursor = 0;
				item_number = 0;
				NumMenu_open(rowOffset, colOffset, buffer, nitem);
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_END:
				num = nitem;
				NumMenu_cursoroff(rowOffset, colOffset, buffer);
				for (i = 0, menu_cursor = 0, item_number = 0; item_number < num && i < num; i++) {
					NumMenu_cursordown(rowOffset, colOffset, nitem, buffer);
				}
				NumMenu_cursoron(rowOffset, colOffset, buffer);
				break;
			case KEY_ENTER:
				CrtNormal(stdscr);
				return item_number;
				break;
			case KEY_CANCEL:
				CrtNormal(stdscr);
				return -1;
			default:
				if (!iscntrl(c)) {
					if (islower(c)) c += 'A' - 'a';
					num = m_search(c, nitem, buffer);
					if (stable_standout) {
						NumMenu_cursoroff(rowOffset, colOffset, buffer);
						CrtReverse(stdscr);
					}
					NumMenu_open(rowOffset, colOffset, buffer, nitem);
					for (menu_cursor = 0, item_number = 0; item_number < num;) {
						NumMenu_cursordown(rowOffset, colOffset, nitem, buffer);
					}
					NumMenu_cursoron(rowOffset, colOffset, buffer);
					break;
				}
			}
	}
}
