/*--------------------------------------------------------------------
	nxeditor
			FILE NAME:undo.c
			Programed by : I.Neva
			R & D  ADVANCED SYSTEMS. IMAGING PRODUCTS.
			1992.06.01
--------------------------------------------------------------------*/
#include "ed.h"

extern char *Realloc();
/*------------------------------------------------------------------------
	Undo buffer stype
		UNDO_DELETE	1
		UNDO_BACKSPACE	2
		UNDO_LINE	3

		BUFFER_NONE	0

		|0||
		|1|string..|
		|3|string..|
		|2|string....|
		|1|string..|
		|1|string..|
		|3|string..|

	..and Ando buffer's List No is UNDO_SYSTEM

------------------------------------------------------------------------*/
/*------------------------------------------------------------------------
	Initialize Undo Buffer
------------------------------------------------------------------------*/
int
InitUndo()
{
	register int	tmpFileno;
	register EditLine *ed;
	char buffer[5];

	tmpFileno = CurrentFileNo;
	CurrentFileNo = UNDO_SYSTEM;
	FileStartInit();
	buffer[0] = BUFFER_NONE+'0';
	buffer[1] = '\0';
	if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
		WarningCheck(MEMORY_ERR_MSG);
		return FALSE;
	} else {
		AppendLast(ed);
	}

	CurrentFileNo = tmpFileno;
	return TRUE;
}

/*------------------------------------------------------------------------
	Undo Mode Set
------------------------------------------------------------------------*/
int
UndoMode(mode)
int mode;
{
	register int	tmpFileno;
	register EditLine *ed;
	char buffer[5];


	tmpFileno = CurrentFileNo;
	CurrentFileNo = UNDO_SYSTEM;

	buffer[0] = mode+'0'	;	/* mode is UNDO_DELETE , UNDO_BACKSPACE or UNDO_LINE */
	buffer[1] = '\0';
	if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
		WarningCheck(MEMORY_ERR_MSG);
		return FALSE;
	} else {
		AppendLast(ed);
	}
	CurrentFileNo = tmpFileno;
	return TRUE;
}

/*------------------------------------------------------------------------
	Store Character to Undo Buffer
------------------------------------------------------------------------*/
int
AddCharToUndoBuffer(c)
int c;
{
	register int tmpFileno;
	register EditLine *ed;
	register int strlength;
	char buffer[MAXEDITLINE+1];

	tmpFileno = CurrentFileNo;
	CurrentFileNo = UNDO_SYSTEM;

	ed = (EditLine *)GetLast();
	strcpy(buffer, ed->buffer);
	strlength = strlen(buffer);
	buffer[strlength] = c;
	buffer[strlength+1] = '\0';
	if (Realloc(ed, buffer) == NULL) {
		WarningCheck(MEMORY_ERR_MSG);
		return FALSE;
	}
	CurrentFileNo = tmpFileno;
	return TRUE;
}

/*------------------------------------------------------------------------
	Store String to Undo Buffer
------------------------------------------------------------------------*/
int
AddStrToUndoBuffer(str)
char *str;
{
	register int tmpFileno;
	register EditLine *ed;
	char buffer[MAXEDITLINE+1];

	tmpFileno = CurrentFileNo;
	CurrentFileNo = UNDO_SYSTEM;

	ed = (EditLine *)GetLast();
	strcpy(buffer, ed->buffer);
	strcat(buffer, str);
	if (Realloc(ed, buffer) == NULL) {
		WarningCheck(MEMORY_ERR_MSG);
		return FALSE;
	}
	CurrentFileNo = tmpFileno;
	return TRUE;
}

/*------------------------------------------------------------------------
	Analize Undo Buffer and Paste
------------------------------------------------------------------------*/
VOID
DoUndo()
{
	register int i;
	register int offset;
	register int tmpFileno;
	register EditLine *ed;
	register int strlength;
	register int mode;
	int c;
	char buffer[MAXEDITLINE+1];
	char *tmpbuffer;

	tmpFileno = CurrentFileNo;
	CurrentFileNo = UNDO_SYSTEM;

	ed = (EditLine *)GetLast();

	mode = ed->buffer[0]-'0';

	if (mode == BUFFER_NONE) {
		CurrentFileNo = tmpFileno;
		return;
	}

	strcpy(buffer, ed->buffer);
	tmpbuffer = buffer;
	tmpbuffer++;
	strcpy(buffer, tmpbuffer);
	strlength = strlen(buffer);
	DeleteList(ed);

	CurrentFileNo = tmpFileno;

	ResetCursorPosition();
	ResetKanjiPosition();
	CrtDrawLine(GetRow(), GetLineOffset(), FALSE);
	switch (mode) {
	case UNDO_DELETE:
		offset = GetBufferOffset();
		for (i = strlength-1; i >= 0; i--) {
			c = buffer[i];
			DefaultInput(c, InputMode_flag, CursorMode, NONE);
			SetBufferOffset(offset);
		}
		SetFileChangeFlag();
		break;
	case UNDO_BACKSPACE:
		for (i = strlength-1; i >= 0; i--) {
			c = buffer[i];
			DefaultInput(c, InputMode_flag, CursorMode, NONE);
		}
		SetFileChangeFlag();
		break;
	case UNDO_LINE:
		if ((ed = (EditLine *)MakeLine(buffer)) == NULL) {
			WarningCheck(MEMORY_ERR_MSG);
		} else {
			InsertLine(GetList(GetLineOffset()-1), ed);
			InsertAndDraw();
		}
		SetFileChangeFlag();
		break;
	}
	CrtDrawLine(GetRow(), GetLineOffset(), TRUE);
}
